@@ -26,8 +26,8 @@
 
 #include "ev-application.h"
 #include "ev-utils.h"
-#include "ev-document-types.h"
 #include "ev-file-helpers.h"
+#include "ev-document-factory.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -36,6 +36,7 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkmain.h>
+#include <libgnomeui/gnome-client.h>
 #include <string.h>
 
 #ifdef ENABLE_DBUS
@@ -45,20 +46,22 @@
 
 G_DEFINE_TYPE (EvApplication, ev_application, G_TYPE_OBJECT);
 
-#define EV_APPLICATION_GET_PRIVATE(object) \
-	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EV_TYPE_APPLICATION, EvApplicationPrivate))
-
 #define APPLICATION_SERVICE_NAME "org.gnome.evince.ApplicationService"
 
 #ifdef ENABLE_DBUS
 gboolean
 ev_application_register_service (EvApplication *application)
 {
-	DBusGConnection *connection;
+	static DBusGConnection *connection = NULL;
 	DBusGProxy *driver_proxy;
 	GError *err = NULL;
 	guint request_name_result;
 
+	if (connection) {
+		g_warning ("Service already registered.");
+		return FALSE;
+	}
+	
 	connection = dbus_g_bus_get (DBUS_BUS_STARTER, &err);
 	if (connection == NULL) {
 		g_warning ("Service registration failed.");
@@ -72,13 +75,25 @@ ev_application_register_service (EvApplication *application)
 						  DBUS_PATH_DBUS,
 						  DBUS_INTERFACE_DBUS);
 
+#if DBUS_VERSION >= 60	
 	if (!org_freedesktop_DBus_request_name (driver_proxy,
                                         	APPLICATION_SERVICE_NAME,
+						DBUS_NAME_FLAG_DO_NOT_QUEUE,
+						&request_name_result, &err)) {
+		g_warning ("Service registration failed.");
+		g_clear_error (&err);
+	}
+#else
+	if (!org_freedesktop_DBus_request_name (driver_proxy,
+						APPLICATION_SERVICE_NAME,
 						0, &request_name_result, &err)) {
 		g_warning ("Service registration failed.");
 		g_clear_error (&err);
 	}
+#endif	
 
+	g_object_unref (driver_proxy);
+	
 	if (request_name_result == DBUS_REQUEST_NAME_REPLY_EXISTS) {
 		return FALSE;
 	}
@@ -111,6 +126,54 @@ ev_application_get_instance (void)
 	return instance;
 }
 
+static void
+removed_from_session (GnomeClient *client, EvApplication *application)
+{
+	ev_application_shutdown (application);
+}
+
+static gint
+save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint shutdown,
+	      GnomeInteractStyle interact_style, gint fast, EvApplication *application)
+{
+	GList *windows, *l;
+	char **restart_argv;
+	int argc = 0, k;
+
+	windows = ev_application_get_windows (application);
+	restart_argv = g_new (char *, g_list_length (windows) + 1);
+	restart_argv[argc++] = g_strdup ("evince");
+
+	for (l = windows; l != NULL; l = l->next) {
+		EvWindow *window = EV_WINDOW (l->data);
+		restart_argv[argc++] = g_strdup (ev_window_get_uri (window));
+	}
+
+	gnome_client_set_restart_command (client, argc, restart_argv);
+
+	for (k = 0; k < argc; k++) {
+		g_free (restart_argv[k]);
+	}
+
+	g_list_free (windows);
+	g_free (restart_argv);
+	
+	return TRUE;
+}
+
+static void
+init_session (EvApplication *application)
+{
+	GnomeClient *client;
+
+	client = gnome_master_client ();
+
+	g_signal_connect (client, "save_yourself",
+			  G_CALLBACK (save_session), application);	
+	g_signal_connect (client, "die",
+			  G_CALLBACK (removed_from_session), application);
+}
+
 gboolean
 ev_application_open_window (EvApplication  *application,
 			    guint32         timestamp,
@@ -120,13 +183,8 @@ ev_application_open_window (EvApplication  *application,
 
 	gtk_widget_show (new_window);
 	
-#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
 	gtk_window_present_with_time (GTK_WINDOW (new_window),
 				      timestamp);
-#else
-	gtk_window_present (GTK_WINDOW (new_window));
-#endif
-
 	return TRUE;
 }
 
@@ -134,17 +192,15 @@ static EvWindow *
 ev_application_get_empty_window (EvApplication *application)
 {
 	EvWindow *empty_window = NULL;
-	GList *windows = gtk_window_list_toplevels ();
+	GList *windows = ev_application_get_windows (application);
 	GList *l;
 
 	for (l = windows; l != NULL; l = l->next) {
-		if (EV_IS_WINDOW (l->data)) {
-			EvWindow *window = EV_WINDOW (l->data);
+		EvWindow *window = EV_WINDOW (l->data);
 
-			if (ev_window_is_empty (window)) {
-				empty_window = window;
-				break;
-			}
+		if (ev_window_is_empty (window)) {
+			empty_window = window;
+			break;
 		}
 	}
 
@@ -167,7 +223,7 @@ ev_application_get_uri_window (EvApplication *application, const char *uri)
 			EvWindow *window = EV_WINDOW (l->data);
 			const char *window_uri = ev_window_get_uri (window);
 
-			if (window_uri && strcmp (window_uri, uri) == 0) {
+			if (window_uri && strcmp (window_uri, uri) == 0 && !ev_window_is_empty (window)) {
 				uri_window = window;
 				break;
 			}
@@ -179,6 +235,42 @@ ev_application_get_uri_window (EvApplication *application, const char *uri)
 	return uri_window;
 }
 
+void
+ev_application_open_uri_at_dest (EvApplication  *application,
+				 const char     *uri,
+				 EvLinkDest     *dest,
+				 guint           timestamp)
+{
+	EvWindow *new_window;
+
+	g_return_if_fail (uri != NULL);
+
+	new_window = ev_application_get_uri_window (application, uri);
+	if (new_window != NULL) {
+		gtk_window_present_with_time (GTK_WINDOW (new_window),
+					      timestamp);
+		if (dest)
+			ev_window_goto_dest (new_window, dest);
+
+		return;
+	}
+
+	new_window = ev_application_get_empty_window (application);
+
+	if (new_window == NULL) {
+		new_window = EV_WINDOW (ev_window_new ());
+	}
+
+	/* We need to load uri before showing the window, so
+	   we can restore window size without flickering */	
+	ev_window_open_uri (new_window, uri, dest);
+
+	gtk_widget_show (GTK_WIDGET (new_window));
+
+	gtk_window_present_with_time (GTK_WINDOW (new_window),
+				      timestamp);
+}
+
 gboolean
 ev_application_open_uri (EvApplication  *application,
 			 const char     *uri,
@@ -186,39 +278,16 @@ ev_application_open_uri (EvApplication  *application,
 			 guint           timestamp,
 			 GError        **error)
 {
-	EvWindow *new_window;
-
-	g_return_val_if_fail (uri != NULL, FALSE);
-
-	new_window = ev_application_get_uri_window (application, uri);
-	if (new_window != NULL) {
-#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
-		gtk_window_present_with_time (GTK_WINDOW (new_window),
-					      timestamp);
-#else
-		gtk_window_present (GTK_WINDOW (new_window));
-#endif	
-		return TRUE;
-	}
-
-	new_window = ev_application_get_empty_window (application);
-
-	if (new_window == NULL) {
-		new_window = EV_WINDOW (ev_window_new ());
-		gtk_widget_show (GTK_WIDGET (new_window));
-	}
 	
-	ev_window_open_uri (new_window, uri);
+	if (page_label && strcmp (page_label, "") != 0) {
+		EvLinkDest *dest;
+		
+		dest = ev_link_dest_new_page_label (page_label);
 
-#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
-	gtk_window_present_with_time (GTK_WINDOW (new_window),
-				      timestamp);
-#else
-	gtk_window_present (GTK_WINDOW (new_window));
-#endif
-
-	if (page_label != NULL) {
-		ev_window_open_page_label (new_window, page_label);
+		ev_application_open_uri_at_dest (application, uri, dest, timestamp);
+		g_object_unref (dest);
+	} else {
+		ev_application_open_uri_at_dest (application, uri, NULL, timestamp);
 	}
 
 	return TRUE;
@@ -233,9 +302,7 @@ ev_application_open_uri_list (EvApplication *application,
 
 	for (l = uri_list; l != NULL; l = l->next) {
 		ev_application_open_uri (application, (char *)l->data,
-					 NULL,
-					 timestamp,
-					 NULL);
+					 NULL, timestamp, NULL);
 	}
 }
 
@@ -253,8 +320,10 @@ ev_application_shutdown (EvApplication *application)
 		g_object_unref (application->recent_model);
 		application->recent_model = NULL;
 	}
-
+	
+	g_free (application->last_chooser_uri);
 	g_object_unref (application);
+	
 	gtk_main_quit ();
 }
 
@@ -266,14 +335,19 @@ ev_application_class_init (EvApplicationClass *ev_application_class)
 static void
 ev_application_init (EvApplication *ev_application)
 {
+	init_session (ev_application);
+
 	ev_application->toolbars_model = egg_toolbars_model_new ();
 
 	ev_application->toolbars_file = g_build_filename
 			(ev_dot_dir (), "evince_toolbar.xml", NULL);
 
-	if (!egg_toolbars_model_load (ev_application->toolbars_model,
+	egg_toolbars_model_load_names (ev_application->toolbars_model,
+				       DATADIR "/evince-toolbar.xml");
+
+	if (!egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
 				      ev_application->toolbars_file)) {
-		egg_toolbars_model_load (ev_application->toolbars_model,
+		egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
 					 DATADIR"/evince-toolbar.xml");
 	}
 
@@ -289,6 +363,25 @@ ev_application_init (EvApplication *ev_application)
     	    	    			    "Evince", NULL);
 }
 
+GList *
+ev_application_get_windows (EvApplication *application)
+{
+	GList *l, *toplevels;
+	GList *windows = NULL;
+
+	toplevels = gtk_window_list_toplevels ();
+
+	for (l = toplevels; l != NULL; l = l->next) {
+		if (EV_IS_WINDOW (l->data)) {
+			windows = g_list_append (windows, l->data);
+		}
+	}
+
+	g_list_free (toplevels);
+
+	return windows;
+}
+
 EggToolbarsModel *ev_application_get_toolbars_model (EvApplication *application)
 {
 	return application->toolbars_model;
@@ -301,8 +394,18 @@ EggRecentModel *ev_application_get_recent_model (EvApplication *application)
 
 void ev_application_save_toolbars_model (EvApplication *application)
 {
-        egg_toolbars_model_save (application->toolbars_model,
-				 application->toolbars_file, "1.0");
+        egg_toolbars_model_save_toolbars (application->toolbars_model,
+			 	          application->toolbars_file, "1.0");
 }
 
+void ev_application_set_chooser_uri (EvApplication *application, const gchar *uri)
+{
+	g_free (application->last_chooser_uri);
+	application->last_chooser_uri = g_strdup (uri);
+}
+
+const gchar* ev_application_get_chooser_uri (EvApplication *application)
+{
+	return application->last_chooser_uri;
+}
 
