@@ -16,13 +16,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-page-action.h,v 1.2 2005/04/01 07:03:05 jrb Exp $
+ *  $Id: ev-page-action.h,v 1.5 2005/05/11 09:42:22 marco Exp $
  */
 
 #ifndef EV_PAGE_ACTION_H
 #define EV_PAGE_ACTION_H
 
 #include <gtk/gtkaction.h>
+#include <gtk/gtktreemodel.h>
 #include <ev-document.h>
 
 G_BEGIN_DECLS
@@ -50,14 +51,18 @@ struct _EvPageActionClass
 {
 	GtkActionClass parent_class;
 
-	void (* goto_page) (EvPageAction *page_action,
-			    int           page_number);
+	void     (* activate_link) (EvPageAction *page_action,
+			            EvLink       *link);
+	gboolean (*activate_label) (EvPageAction *page_action,
+				    char         *label);
 };
 
-GType ev_page_action_get_type         (void);
-void  ev_page_action_set_document     (EvPageAction *page_action,
-				       EvDocument   *document);
-
+GType ev_page_action_get_type     (void);
+void  ev_page_action_set_document (EvPageAction *page_action,
+				   EvDocument   *document);
+void  ev_page_action_set_model    (EvPageAction *page_action,
+				   GtkTreeModel *model);
+void  ev_page_action_grab_focus   (EvPageAction *page_action);
 
 G_END_DECLS
 
